#!/bin/bash

while [ -n "$1" ]
    do
        case "$1" in
            --cmdid) IR_ID_ADDR="52$(printf "%02d" $2)"
                     IR_SIZE_ADDR="54$(printf "%02d" $2)"
                    shift ;;
            --addr) DEV_ADDR=$2
                    shift ;;
            --port) PORT=$2
                    shift ;;
            --no_overwrite)
                    NO_OVERWRITE=true ;;
            --data) IFS=', ' read -r -a IR_CODE <<< "$(echo $2 | base64 -d | xxd -i | tr -d '\n')"
                    shift ;;
            *) exit 2 ;;
        esac
        shift
    done

modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x03 -o100 -r0x80 -c1 || exit 1

for ((idx=0; idx<${#IR_CODE[@]}; idx=idx+2)); do
    let "idx2=idx + 1"
    NUM=0x${IR_CODE[$idx]#"0x"}${IR_CODE[$idx2]#"0x"}
    TWO_BYTE+=($NUM)
done


NON_ZERO_REGISTERS_COUNT=0
for (( idx=${#TWO_BYTE[@]}-1 ; idx>=0 ; idx-- )) ; do
    DEC=$(printf "%d" "${TWO_BYTE[idx]}")
    if [[ "$DEC" -ne "0" ]]; then
        let "NON_ZERO_REGISTERS_COUNT = $idx + 1"
        break
    fi
done

if [ "$NO_OVERWRITE" = true ]; then
    WRITED_COMMAND_SIZE_HEX="$(modbus_client --debug -mrtu -pnone -s2 ${PORT} -a$DEV_ADDR -t0x04 -r$IR_SIZE_ADDR -c 1 | grep Data: | cut -c7- )"
    WRITED_COMMAND_SIZE=$(printf "%d" $WRITED_COMMAND_SIZE_HEX)
    let "LEGACY_WRITED_COMMAND_SIZE = $WRITED_COMMAND_SIZE / 2"

    if [[ "$WRITED_COMMAND_SIZE" -eq "$NON_ZERO_REGISTERS_COUNT" || "$LEGACY_WRITED_COMMAND_SIZE" -eq "$NON_ZERO_REGISTERS_COUNT" ]]; then
        exit 0
    fi
fi

EMPTY_DATA="0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0"
modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x0F -o5000 -r5200 -c32 ${EMPTY_DATA} || exit 2
modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x0F -o5000 -r5001 -c2 0 0 || exit 2
modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x0F -o5000 -r5100 -c32 ${EMPTY_DATA} || exit 2
modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x0F -o5000 -r5300 -c32 ${EMPTY_DATA} || exit 2

safe_exit() {
    # Если открыли режим записи, нужно обязательно его закрыть!!!
    for ((i = 0; i < 10; i++)); do
        modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x05 -o5000 -r$IR_ID_ADDR 0 && break
    done

    exit $1
}

TWO_BYTE+=(0x0000)
TWO_BYTE+=(0x0000)

echo IR_ID_ADDR $IR_ID_ADDR
echo DEV_ADDR $DEV_ADDR
echo PORT $PORT
echo TWO_BYTE ${TWO_BYTE[*]}

echo SIZE ${#TWO_BYTE[@]}
echo NON_ZERO_REGISTERS_COUNT $NON_ZERO_REGISTERS_COUNT


PART_SIZE=100
let "WHOLE_PART_COUNT=${#TWO_BYTE[@]} / PART_SIZE"
let "LAST_PART_SIZE=${#TWO_BYTE[@]} % PART_SIZE"

RAM_START_REG=2000

modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x05 -o5000 -r$IR_ID_ADDR 1 || safe_exit 3
START=0

if [[ "$PART_SIZE" -le "${#TWO_BYTE[@]}" ]]; then
    for ((i = 1; i <= WHOLE_PART_COUNT; i++)); do
        echo START $START
        modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x10 -o5000 -r$RAM_START_REG -c $PART_SIZE "${TWO_BYTE[@]:$START:$PART_SIZE}" || safe_exit 4
        let "(RAM_START_REG = $RAM_START_REG + $PART_SIZE)"
        let "START= $i * $PART_SIZE"
    done
fi

if [[ "$LAST_PART_SIZE" -gt 0 ]]; then
    modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x10 -o5000 -r$RAM_START_REG -c $LAST_PART_SIZE "${TWO_BYTE[@]:$START:$LAST_PART_SIZE}" || safe_exit 5
fi


modbus_client --debug -mrtu -pnone -s2 "${PORT}" -a$DEV_ADDR -t0x05 -o5000 -r$IR_ID_ADDR 0 || safe_exit 6
WRITED_COMMAND_SIZE_HEX="$(modbus_client --debug -mrtu -pnone -s2 ${PORT} -a$DEV_ADDR -t0x04 -r$IR_SIZE_ADDR -c 1 | grep Data: | cut -c7- )"
WRITED_COMMAND_SIZE=$(printf "%d" $WRITED_COMMAND_SIZE_HEX)
let "LEGACY_WRITED_COMMAND_SIZE = $WRITED_COMMAND_SIZE / 2"

if [[ "$WRITED_COMMAND_SIZE" -ne "$NON_ZERO_REGISTERS_COUNT" && "$LEGACY_WRITED_COMMAND_SIZE" -ne "$NON_ZERO_REGISTERS_COUNT" ]]; then
    exit 7
fi

echo Success!
